@@ -113,6 +113,23 @@ gnutls_cipher_add_auth (gnutls_cipher_hd_t handle, const void *text, size_t text
   return 0;
 }
 
+/**
+ * gnutls_cipher_set_iv:
+ * @handle: is a #gnutls_cipher_hd_t structure.
+ * @iv: the IV to set
+ * @ivlen: The length of the IV
+ *
+ * This function will set the IV to be used for the next
+ * encryption block.
+ *
+ * Since: 2.99.0
+ **/
+void
+gnutls_cipher_set_iv (gnutls_cipher_hd_t handle, void *iv, size_t ivlen)
+{
+  _gnutls_cipher_setiv((cipher_hd_st *)handle, iv, ivlen);
+}
+
 /**
  * gnutls_cipher_encrypt:
  * @handle: is a #gnutls_cipher_hd_t structure.
@@ -169,7 +186,7 @@ gnutls_cipher_decrypt (gnutls_cipher_hd_t handle, void *ciphertext,
  * Since: 2.10.0
  **/
 int
-gnutls_cipher_encrypt2 (gnutls_cipher_hd_t handle, void *text, size_t textlen,
+gnutls_cipher_encrypt2 (gnutls_cipher_hd_t handle, const void *text, size_t textlen,
                         void *ciphertext, size_t ciphertextlen)
 {
   return _gnutls_cipher_encrypt2 ((cipher_hd_st *) handle, text, textlen,
