@@ -25,7 +25,7 @@
 #include <gnutls_int.h>
 #include <gnutls_errors.h>
 #include <gnutls_datum.h>
-#include <gnutls_algorithms.h>
+#include <algorithms.h>
 #include <gnutls_handshake.h>
 #include <gnutls_num.h>
 #include <gnutls_constate.h>
@@ -124,7 +124,7 @@ decrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
   cipher_hd_st cipher_hd;
   gnutls_datum_t key, IV, mac_secret, state;
   opaque final[MAC_SECRET_SIZE];
-  time_t timestamp = time (0);
+  time_t timestamp = gnutls_time (0);
   int ret;
 
   /* Check the integrity of ticket using HMAC-SHA-256. */
