@@ -31,7 +31,7 @@
 #include <gnutls_constate.h>
 #include <gnutls_errors.h>
 #include <gnutls_kx.h>
-#include <gnutls_algorithms.h>
+#include <algorithms.h>
 #include <gnutls_num.h>
 #include <gnutls_datum.h>
 #include <gnutls_state.h>
