@@ -328,6 +328,10 @@ static const gnutls_error_entry error_algorithms[] = {
                GNUTLS_E_TIMEDOUT, 1),
   ERROR_ENTRY (N_("The operation was cancelled due to user error"),
                GNUTLS_E_USER_ERROR, 1),
+  ERROR_ENTRY (N_("No supported ECC curves were found"),
+               GNUTLS_E_ECC_NO_SUPPORTED_CURVES, 1),
+  ERROR_ENTRY (N_("The curve is unsupported"),
+               GNUTLS_E_ECC_UNSUPPORTED_CURVE, 1),
   {NULL, NULL, 0, 0}
 };
 
@@ -574,6 +578,30 @@ _gnutls_log (int level, const char *fmt, ...)
     }
 }
 
+void
+_gnutls_audit_log (gnutls_session_t session, const char *fmt, ...)
+{
+  va_list args;
+  char *str;
+  int ret;
+
+  if (_gnutls_audit_log_func == NULL && _gnutls_log_func == NULL)
+    return;
+
+  va_start (args, fmt);
+  ret = vasprintf (&str, fmt, args);
+  va_end (args);
+
+  if (ret >= 0)
+    {
+      if (_gnutls_audit_log_func)
+        _gnutls_audit_log_func (session, str);
+      else
+        _gnutls_log_func(1, str);
+      free (str);
+    }
+}
+
 #ifndef DEBUG
 #ifndef C99_MACROS
 
