@@ -39,6 +39,7 @@
 #include <ext/safe_renegotiation.h>
 #include <ext/signature.h>
 #include <ext/safe_renegotiation.h>
+#include <ext/ecc.h>
 #include <gnutls_num.h>
 
 
@@ -153,7 +154,7 @@ _gnutls_parse_extensions (gnutls_session_t session,
   if (session->security_parameters.entity == GNUTLS_CLIENT)
     for (i = 0; i < session->internals.extensions_sent_size; i++)
       {
-        _gnutls_debug_log ("EXT[%d]: expecting extension '%s'\n",
+        _gnutls_handshake_log ("EXT[%d]: expecting extension '%s'\n",
                            session,
                            _gnutls_extension_get_name
                            (session->internals.extensions_sent[i]));
@@ -173,7 +174,7 @@ _gnutls_parse_extensions (gnutls_session_t session,
       pos += 2;
 
 #if 0
-      _gnutls_debug_log ("EXT[%p]: Found extension '%s/%d'\n", session,
+      _gnutls_handshake_log ("EXT[%p]: Found extension '%s/%d'\n", session,
                          _gnutls_extension_get_name (type), type);
 #endif
 
@@ -195,7 +196,7 @@ _gnutls_parse_extensions (gnutls_session_t session,
       if (ext_recv == NULL)
         continue;
 
-      _gnutls_debug_log ("EXT[%p]: Parsing extension '%s/%d' (%d bytes)\n",
+      _gnutls_handshake_log ("EXT[%p]: Parsing extension '%s/%d' (%d bytes)\n",
                          session, _gnutls_extension_get_name (type), type,
                          size);
 
@@ -230,7 +231,7 @@ _gnutls_extension_list_add (gnutls_session_t session, uint16_t type)
         }
       else
         {
-          _gnutls_debug_log ("extensions: Increase MAX_EXT_TYPES\n");
+          _gnutls_handshake_log ("extensions: Increase MAX_EXT_TYPES\n");
         }
     }
 }
@@ -281,7 +282,7 @@ _gnutls_gen_extensions (gnutls_session_t session, gnutls_buffer_st * extdata,
            */
           _gnutls_extension_list_add (session, p->type);
 
-          _gnutls_debug_log ("EXT[%p]: Sending extension %s (%d bytes)\n",
+          _gnutls_handshake_log ("EXT[%p]: Sending extension %s (%d bytes)\n",
                              session, p->name, size);
         }
       else if (size < 0)
@@ -334,6 +335,14 @@ _gnutls_ext_init (void)
   if (ret != GNUTLS_E_SUCCESS)
     return ret;
 
+  ret = _gnutls_ext_register (&ext_mod_supported_ecc);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
+
+  ret = _gnutls_ext_register (&ext_mod_supported_ecc_pf);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
+
   ret = _gnutls_ext_register (&ext_mod_sig);
   if (ret != GNUTLS_E_SUCCESS)
     return ret;
