@@ -32,6 +32,7 @@
 
 #include <gnutls_extensions.h>  /* for _gnutls_ext_init */
 #include <locks.h>
+#include <system.h>
 #include <accelerated/cryptodev.h>
 #include <accelerated/accelerated.h>
 
@@ -49,6 +50,7 @@ ASN1_TYPE _gnutls_pkix1_asn;
 ASN1_TYPE _gnutls_gnutls_asn;
 
 gnutls_log_func _gnutls_log_func;
+gnutls_audit_log_func _gnutls_audit_log_func;
 int _gnutls_log_level = 0;      /* default log level */
 
 /**
@@ -69,6 +71,40 @@ gnutls_global_set_log_function (gnutls_log_func log_func)
   _gnutls_log_func = log_func;
 }
 
+/**
+ * gnutls_global_set_audit_log_function:
+ * @log_func: it is the audit log function
+ *
+ * This is the function where you set the logging function gnutls is
+ * going to use.  This is different from gnutls_global_set_log_function()
+ * because it will report the session of the event if any. Note that
+ * that session might be null if there is no corresponding TLS session.
+ *
+ * gnutls_audit_log_func is of the form,
+ * void (*gnutls_audit_log_func)( gnutls_session_t, int level, const char*);
+ **/
+void
+gnutls_global_set_audit_log_function (gnutls_audit_log_func log_func)
+{
+  _gnutls_audit_log_func = log_func;
+}
+
+/**
+ * gnutls_global_set_time_function:
+ * @time_func: it's the system time function
+ *
+ * This is the function where you can override the default system
+ * time function.
+ *
+ * gnutls_time_func is of the form,
+ * time_t (*gnutls_time_func)( time*);
+ **/
+void
+gnutls_global_set_time_function (gnutls_time_func time_func)
+{
+  gnutls_time = time_func;
+}
+
 /**
  * gnutls_global_set_log_level:
  * @level: it's an integer from 0 to 9.
@@ -137,7 +173,6 @@ gnutls_global_set_mem_functions (gnutls_alloc_function alloc_func,
 
 static int _gnutls_init = 0;
 
-
 /**
  * gnutls_global_init:
  *
