@@ -30,7 +30,7 @@
 #include "gnutls_errors.h"
 #include "gnutls_dh.h"
 #include "debug.h"
-#include "gnutls_algorithms.h"
+#include "algorithms.h"
 #include "gnutls_compress.h"
 #include "gnutls_cipher.h"
 #include "gnutls_buffers.h"
@@ -104,7 +104,7 @@ _gnutls_handshake_hash_buffers_clear (gnutls_session_t session)
   session->security_parameters.handshake_mac_handle_type = 0;
   session->internals.handshake_mac_handle_init = 0;
 
-  _gnutls_handshake_hash_buffer_clear (session);
+  _gnutls_buffer_clear(&session->internals.handshake_hash_buffer);
 }
 
 /* this will copy the required values for resuming to
@@ -338,7 +338,7 @@ _gnutls_tls_create_random (opaque * dst)
    * system's time.
    */
 
-  tim = time (NULL);
+  tim = gnutls_time (NULL);
   /* generate server random value */
   _gnutls_write_uint32 (tim, dst);
 
@@ -454,7 +454,7 @@ _gnutls_read_client_hello (gnutls_session_t session, opaque * data,
   _gnutls_tls_create_random (rnd);
   _gnutls_set_server_random (session, rnd);
 
-  session->security_parameters.timestamp = time (NULL);
+  session->security_parameters.timestamp = gnutls_time (NULL);
 
   DECR_LEN (len, 1);
   session_id_len = data[pos++];
@@ -1336,7 +1336,7 @@ _gnutls_recv_handshake (gnutls_session_t session,
         }
 
        if (ret == GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET)
-        _gnutls_audit_log("Received unexpected handshake message '%s' (%d). Expected '%s' (%d)\n",
+        _gnutls_audit_log(session, "Received unexpected handshake message '%s' (%d). Expected '%s' (%d)\n",
          _gnutls_handshake2str(hsk.htype), (int)hsk.htype, _gnutls_handshake2str(type), (int)type);
 
       return gnutls_assert_val_fatal(ret);
@@ -1968,7 +1968,7 @@ _gnutls_send_client_hello (gnutls_session_t session, int again)
 
       /* In order to know when this session was initiated.
        */
-      session->security_parameters.timestamp = time (NULL);
+      session->security_parameters.timestamp = gnutls_time (NULL);
 
       /* Generate random data 
        */
@@ -3344,6 +3344,15 @@ _gnutls_remove_unwanted_ciphersuites (gnutls_session_t session,
           if (server)
             delete = check_server_params (session, kx, alg, alg_size);
         }
+       
+      /* If we have not agreed to a common curve with the peer don't bother
+       * negotiating ECDH.
+       */
+      if (session->security_parameters.entity == GNUTLS_SERVER && _gnutls_session_is_ecc(session))
+        {
+          if (_gnutls_session_ecc_curve_get(session) == GNUTLS_ECC_CURVE_INVALID)
+            delete = 1;
+        }
 
       /* These two SRP kx's are marked to require a CRD_CERTIFICATE,
          (see cred_mappings in gnutls_algorithms.c), but it also
