@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Free Software Foundation
+ * Copyright (C) 2009,2011 Free Software Foundation
  *
  * Author: Jonathan Bastien-Filiatrault
  *
@@ -54,10 +54,10 @@
  * Cost: O(1)
  */
 void
-_mbuffer_init (mbuffer_head_st * buf)
+_mbuffer_head_init (mbuffer_head_st * buf)
 {
   buf->head = NULL;
-  buf->tail = &buf->head;
+  buf->tail = NULL;
 
   buf->length = 0;
   buf->byte_length = 0;
@@ -69,7 +69,7 @@ _mbuffer_init (mbuffer_head_st * buf)
  * n: Number of segments currently in the buffer.
  */
 void
-_mbuffer_clear (mbuffer_head_st * buf)
+_mbuffer_head_clear (mbuffer_head_st * buf)
 {
   mbuffer_st *bufel, *next;
 
@@ -79,7 +79,7 @@ _mbuffer_clear (mbuffer_head_st * buf)
       gnutls_free (bufel);
     }
 
-  _mbuffer_init (buf);
+  _mbuffer_head_init (buf);
 }
 
 /* Append a segment to the end of this buffer.
@@ -90,12 +90,67 @@ void
 _mbuffer_enqueue (mbuffer_head_st * buf, mbuffer_st * bufel)
 {
   bufel->next = NULL;
+  bufel->prev = buf->tail;
 
   buf->length++;
   buf->byte_length += bufel->msg.size - bufel->mark;
 
-  *(buf->tail) = bufel;
-  buf->tail = &bufel->next;
+  if (buf->tail != NULL)
+    buf->tail->next = bufel;
+  else
+    buf->head = bufel;
+  buf->tail = bufel;
+}
+
+/* Remove a segment from the buffer.
+ *
+ * Cost: O(1)
+ *
+ * Returns the buffer following it.
+ */
+mbuffer_st *
+_mbuffer_dequeue (mbuffer_head_st * buf, mbuffer_st * bufel)
+{
+mbuffer_st* ret = bufel->next;
+
+  if (buf->tail == bufel) /* if last */
+    buf->tail = bufel->prev;
+  
+  if (buf->head == bufel) /* if first */
+    buf->head = bufel->next;
+
+  if (bufel->prev)
+    bufel->prev->next = bufel->next;
+
+  if (bufel->next)
+    bufel->next->prev = NULL;
+
+  buf->length--;
+  buf->byte_length -= bufel->msg.size - bufel->mark;
+  
+  bufel->next = bufel->prev = NULL;
+  
+  return ret;
+}
+
+/* Get a reference to the first segment of the buffer and
+ * remove it from the list.
+ *
+ * Used to start iteration.
+ *
+ * Cost: O(1)
+ */
+mbuffer_st *
+_mbuffer_head_pop_first (mbuffer_head_st * buf)
+{
+  mbuffer_st *bufel = buf->head;
+
+  if (buf->head == NULL)
+    return NULL;
+
+  _mbuffer_dequeue(buf, bufel);
+    
+  return bufel;
 }
 
 /* Get a reference to the first segment of the buffer and its data.
@@ -105,19 +160,22 @@ _mbuffer_enqueue (mbuffer_head_st * buf, mbuffer_st * bufel)
  * Cost: O(1)
  */
 mbuffer_st *
-_mbuffer_get_first (mbuffer_head_st * buf, gnutls_datum_t * msg)
+_mbuffer_head_get_first (mbuffer_head_st * buf, gnutls_datum_t * msg)
 {
   mbuffer_st *bufel = buf->head;
 
-  if (bufel)
+  if (msg)
     {
-      msg->data = bufel->msg.data + bufel->mark;
-      msg->size = bufel->msg.size - bufel->mark;
-    }
-  else
-    {
-      msg->data = NULL;
-      msg->size = 0;
+      if (bufel)
+        {
+          msg->data = bufel->msg.data + bufel->mark;
+          msg->size = bufel->msg.size - bufel->mark;
+      }
+    else
+      {
+        msg->data = NULL;
+        msg->size = 0;
+      }
     }
   return bufel;
 }
@@ -129,19 +187,22 @@ _mbuffer_get_first (mbuffer_head_st * buf, gnutls_datum_t * msg)
  * Cost: O(1)
  */
 mbuffer_st *
-_mbuffer_get_next (mbuffer_st * cur, gnutls_datum_t * msg)
+_mbuffer_head_get_next (mbuffer_st * cur, gnutls_datum_t * msg)
 {
   mbuffer_st *bufel = cur->next;
 
-  if (bufel)
+  if (msg)
     {
-      msg->data = bufel->msg.data + bufel->mark;
-      msg->size = bufel->msg.size - bufel->mark;
-    }
-  else
-    {
-      msg->data = NULL;
-      msg->size = 0;
+      if (bufel)
+        {
+          msg->data = bufel->msg.data + bufel->mark;
+          msg->size = bufel->msg.size - bufel->mark;
+        }
+      else
+        {
+          msg->data = NULL;
+          msg->size = 0;
+        }
     }
   return bufel;
 }
@@ -156,36 +217,31 @@ _mbuffer_get_next (mbuffer_st * cur, gnutls_datum_t * msg)
 static inline void
 remove_front (mbuffer_head_st * buf)
 {
-  mbuffer_st *bufel;
+  mbuffer_st *bufel = buf->head;
 
-  if (!buf->head)
+  if (!bufel)
     return;
 
-  bufel = buf->head;
-  buf->head = bufel->next;
-
-  buf->byte_length -= (bufel->msg.size - bufel->mark);
-  buf->length -= 1;
+  _mbuffer_dequeue(buf, bufel);
   gnutls_free (bufel);
-
-  if (!buf->head)
-    buf->tail = &buf->head;
 }
 
 /* Remove a specified number of bytes from the start of the buffer.
  *
  * Useful for uses that treat the buffer as a simple array of bytes.
  *
- * Returns 0 on success or an error code otherwise.
+ * If more than one mbuffer_st have been removed it
+ * returns 1, 0 otherwise and an error code on error.
  *
  * Cost: O(n)
  * n: Number of segments needed to remove the specified amount of data.
  */
 int
-_mbuffer_remove_bytes (mbuffer_head_st * buf, size_t bytes)
+_mbuffer_head_remove_bytes (mbuffer_head_st * buf, size_t bytes)
 {
   size_t left = bytes;
   mbuffer_st *bufel, *next;
+  int ret = 0;
 
   if (bytes > buf->byte_length)
     {
@@ -201,6 +257,7 @@ _mbuffer_remove_bytes (mbuffer_head_st * buf, size_t bytes)
         {
           left -= (bufel->msg.size - bufel->mark);
           remove_front (buf);
+          ret = 1;
         }
       else
         {
@@ -210,7 +267,7 @@ _mbuffer_remove_bytes (mbuffer_head_st * buf, size_t bytes)
         }
     }
 
-  return 0;
+  return ret;
 }
 
 /* Allocate a buffer segment. The segment is not initially "owned" by
@@ -230,19 +287,16 @@ _mbuffer_alloc (size_t payload_size, size_t maximum_size)
 {
   mbuffer_st *st;
 
-  st = gnutls_malloc (maximum_size + sizeof (mbuffer_st));
+  st = gnutls_calloc (1, maximum_size + sizeof (mbuffer_st));
   if (st == NULL)
     {
       gnutls_assert ();
       return NULL;
     }
 
-  //payload points after the mbuffer_st structure
+  /* payload points after the mbuffer_st structure */
   st->msg.data = (opaque *) st + sizeof (mbuffer_st);
   st->msg.size = payload_size;
-  st->mark = 0;
-  st->user_mark = 0;
-  st->next = NULL;
   st->maximum_size = maximum_size;
 
   return st;
@@ -301,14 +355,14 @@ _mbuffer_linearize (mbuffer_head_st * buf)
       return GNUTLS_E_MEMORY_ERROR;
     }
 
-  for (cur = _mbuffer_get_first (buf, &msg);
-       msg.data != NULL; cur = _mbuffer_get_next (cur, &msg))
+  for (cur = _mbuffer_head_get_first (buf, &msg);
+       msg.data != NULL; cur = _mbuffer_head_get_next (cur, &msg))
     {
       memcpy (&bufel->msg.data[pos], msg.data, cur->msg.size);
       pos += cur->msg.size;
     }
 
-  _mbuffer_clear (buf);
+  _mbuffer_head_clear (buf);
   _mbuffer_enqueue (buf, bufel);
 
   return 0;
