@@ -95,7 +95,7 @@ gnutls_privkey_get_pk_algorithm (gnutls_privkey_t key, unsigned int *bits)
       return gnutls_pkcs11_privkey_get_pk_algorithm (key->key.pkcs11, bits);
     case GNUTLS_PRIVKEY_X509:
       if (bits)
-        *bits = _gnutls_mpi_get_nbits (key->key.x509->params[0]);
+        *bits = _gnutls_mpi_get_nbits (key->key.x509->params.params[0]);
       return gnutls_x509_privkey_get_pk_algorithm (key->key.x509);
     default:
       gnutls_assert ();
@@ -106,27 +106,20 @@ gnutls_privkey_get_pk_algorithm (gnutls_privkey_t key, unsigned int *bits)
 
 static int
 privkey_to_pubkey (gnutls_pk_algorithm_t pk,
-                   const bigint_t * params, int params_size,
-                   bigint_t * new_params, int *new_params_size)
+                   const gnutls_pk_params_st* priv,
+                   gnutls_pk_params_st* pub)
 {
-  int ret, i;
+  int ret;
 
   switch (pk)
     {
     case GNUTLS_PK_RSA:
-      if (*new_params_size < RSA_PUBLIC_PARAMS
-          || params_size < RSA_PRIVATE_PARAMS)
-        {
-          gnutls_assert ();
-          return GNUTLS_E_INVALID_REQUEST;
-        }
+      pub->params[0] = _gnutls_mpi_copy (priv->params[0]);
+      pub->params[1] = _gnutls_mpi_copy (priv->params[1]);
 
-      new_params[0] = _gnutls_mpi_copy (params[0]);
-      new_params[1] = _gnutls_mpi_copy (params[1]);
+      pub->params_nr = RSA_PUBLIC_PARAMS;
 
-      *new_params_size = RSA_PUBLIC_PARAMS;
-
-      if (new_params[0] == NULL || new_params[1] == NULL)
+      if (pub->params[0] == NULL || pub->params[1] == NULL)
         {
           gnutls_assert ();
           ret = GNUTLS_E_MEMORY_ERROR;
@@ -135,22 +128,38 @@ privkey_to_pubkey (gnutls_pk_algorithm_t pk,
 
       break;
     case GNUTLS_PK_DSA:
-      if (*new_params_size < DSA_PUBLIC_PARAMS
-          || params_size < DSA_PRIVATE_PARAMS)
+      pub->params[0] = _gnutls_mpi_copy (priv->params[0]);
+      pub->params[1] = _gnutls_mpi_copy (priv->params[1]);
+      pub->params[2] = _gnutls_mpi_copy (priv->params[2]);
+      pub->params[3] = _gnutls_mpi_copy (priv->params[3]);
+
+      pub->params_nr = DSA_PUBLIC_PARAMS;
+
+      if (pub->params[0] == NULL || pub->params[1] == NULL ||
+          pub->params[2] == NULL || pub->params[3] == NULL)
         {
           gnutls_assert ();
-          return GNUTLS_E_INVALID_REQUEST;
+          ret = GNUTLS_E_MEMORY_ERROR;
+          goto cleanup;
         }
 
-      new_params[0] = _gnutls_mpi_copy (params[0]);
-      new_params[1] = _gnutls_mpi_copy (params[1]);
-      new_params[2] = _gnutls_mpi_copy (params[2]);
-      new_params[3] = _gnutls_mpi_copy (params[3]);
+      break;
+    case GNUTLS_PK_ECC:
+      pub->params[0] = _gnutls_mpi_copy (priv->params[0]);
+      pub->params[1] = _gnutls_mpi_copy (priv->params[1]);
+      pub->params[2] = _gnutls_mpi_copy (priv->params[2]);
+      pub->params[3] = _gnutls_mpi_copy (priv->params[3]);
+      pub->params[4] = _gnutls_mpi_copy (priv->params[4]);
+      pub->params[5] = _gnutls_mpi_copy (priv->params[5]);
+      pub->params[6] = _gnutls_mpi_copy (priv->params[6]);
 
-      *new_params_size = DSA_PUBLIC_PARAMS;
+      pub->params_nr = ECC_PUBLIC_PARAMS;
+      pub->flags = priv->flags;
 
-      if (new_params[0] == NULL || new_params[1] == NULL ||
-          new_params[2] == NULL || new_params[3] == NULL)
+      if (pub->params[0] == NULL || pub->params[1] == NULL ||
+          pub->params[2] == NULL || pub->params[3] == NULL ||
+          pub->params[4] == NULL || pub->params[5] == NULL ||
+          pub->params[6] == NULL)
         {
           gnutls_assert ();
           ret = GNUTLS_E_MEMORY_ERROR;
@@ -165,8 +174,7 @@ privkey_to_pubkey (gnutls_pk_algorithm_t pk,
 
   return 0;
 cleanup:
-  for (i = 0; i < *new_params_size; i++)
-    _gnutls_mpi_release (new_params[i]);
+  gnutls_pk_params_release(pub);
   return ret;
 }
 
@@ -175,7 +183,7 @@ cleanup:
  */
 int
 _gnutls_privkey_get_public_mpis (gnutls_privkey_t key,
-                                 bigint_t * params, int *params_size)
+                                 gnutls_pk_params_st * params)
 {
   int ret;
   gnutls_pk_algorithm_t pk = gnutls_privkey_get_pk_algorithm (key, NULL);
@@ -185,9 +193,8 @@ _gnutls_privkey_get_public_mpis (gnutls_privkey_t key,
 #ifdef ENABLE_OPENPGP
     case GNUTLS_PRIVKEY_OPENPGP:
       {
-        bigint_t tmp_params[MAX_PRIV_PARAMS_SIZE];
-        int tmp_params_size = MAX_PRIV_PARAMS_SIZE;
-        uint32_t kid[2], i;
+        gnutls_pk_params_st tmp_params;
+        uint32_t kid[2];
         uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
 
         ret =
@@ -197,13 +204,11 @@ _gnutls_privkey_get_public_mpis (gnutls_privkey_t key,
           {
             KEYID_IMPORT (kid, keyid);
             ret = _gnutls_openpgp_privkey_get_mpis (key->key.openpgp, kid,
-                                                    tmp_params,
-                                                    &tmp_params_size);
+                                                    &tmp_params);
           }
         else
           ret = _gnutls_openpgp_privkey_get_mpis (key->key.openpgp, NULL,
-                                                  tmp_params,
-                                                  &tmp_params_size);
+                                                  &tmp_params);
 
         if (ret < 0)
           {
@@ -212,21 +217,18 @@ _gnutls_privkey_get_public_mpis (gnutls_privkey_t key,
           }
 
         ret = privkey_to_pubkey (pk,
-                                 tmp_params, tmp_params_size,
-                                 params, params_size);
-
-        for (i = 0; i < tmp_params_size; i++)
-          _gnutls_mpi_release (&tmp_params[i]);
+                                 &tmp_params,
+                                 params);
 
+        gnutls_pk_params_release(&tmp_params);
       }
 
       break;
 #endif
     case GNUTLS_PRIVKEY_X509:
       ret = privkey_to_pubkey (pk,
-                               key->key.x509->params,
-                               key->key.x509->params_size, params,
-                               params_size);
+                               &key->key.x509->params,
+                               params);
       break;
     default:
       gnutls_assert ();
@@ -574,8 +576,8 @@ _gnutls_privkey_sign_hash (gnutls_privkey_t key,
                                                hash, signature);
     case GNUTLS_PRIVKEY_X509:
       return _gnutls_soft_sign (key->key.x509->pk_algorithm,
-                                key->key.x509->params,
-                                key->key.x509->params_size, hash, signature);
+                                &key->key.x509->params,
+                                hash, signature);
     default:
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
@@ -616,8 +618,8 @@ gnutls_privkey_decrypt_data (gnutls_privkey_t key,
 #endif
     case GNUTLS_PRIVKEY_X509:
       return _gnutls_pkcs1_rsa_decrypt (plaintext, ciphertext,
-                                        key->key.x509->params,
-                                        key->key.x509->params_size, 2);
+                                        &key->key.x509->params,
+                                        2);
     case GNUTLS_PRIVKEY_PKCS11:
       return _gnutls_pkcs11_privkey_decrypt_data (key->key.pkcs11,
                                                  flags,
