@@ -27,7 +27,7 @@
 #include <gnutls_errors.h>
 #include <x509_b64.h>
 #include <auth/cert.h>
-#include <gnutls_algorithms.h>
+#include <algorithms.h>
 #include <gnutls_datum.h>
 #include <gnutls_mpi.h>
 #include <gnutls_global.h>
@@ -133,6 +133,7 @@ _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_pcert_st* cert,
         }
       break;
     case GNUTLS_PK_DSA:
+    case GNUTLS_PK_ECC:
       _gnutls_hash_deinit (&td_sha, concat);
 
       if ((hash_algo != GNUTLS_DIG_SHA1) && (hash_algo != GNUTLS_DIG_SHA224)
@@ -166,8 +167,8 @@ _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_pcert_st* cert,
  * given data. The output will be allocated and be put in signature.
  */
 int
-_gnutls_soft_sign (gnutls_pk_algorithm_t algo, bigint_t * params,
-                   int params_size, const gnutls_datum_t * data,
+_gnutls_soft_sign (gnutls_pk_algorithm_t algo, gnutls_pk_params_st * params,
+                   const gnutls_datum_t * data,
                    gnutls_datum_t * signature)
 {
   int ret;
@@ -176,26 +177,21 @@ _gnutls_soft_sign (gnutls_pk_algorithm_t algo, bigint_t * params,
     {
     case GNUTLS_PK_RSA:
       /* encrypt */
-      if ((ret = _gnutls_pkcs1_rsa_encrypt (signature, data, params,
-                                            params_size, 1)) < 0)
+      if ((ret = _gnutls_pkcs1_rsa_encrypt (signature, data, params, 1)) < 0)
         {
           gnutls_assert ();
           return ret;
         }
 
       break;
-    case GNUTLS_PK_DSA:
-      /* sign */
-      if ((ret = _gnutls_dsa_sign (signature, data, params, params_size)) < 0)
+    default:
+      ret = _gnutls_pk_sign( algo, signature, data, params);
+      if (ret < 0)
         {
           gnutls_assert ();
           return ret;
         }
       break;
-    default:
-      gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
-      break;
     }
 
   return 0;
@@ -290,6 +286,7 @@ verify_tls_hash (gnutls_protocol_t ver, gnutls_pcert_st* cert,
 
       break;
     case GNUTLS_PK_DSA:
+    case GNUTLS_PK_ECC:
 
       vdata.data = &hash_concat->data[sha1pos];
       vdata.size = hash_concat->size - sha1pos;
@@ -728,6 +725,7 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
       dconcat.size = 36;
       break;
     case GNUTLS_PK_DSA:
+    case GNUTLS_PK_ECC:
 
       dconcat.data = &concat[16];
       dconcat.size = 20;
@@ -747,27 +745,11 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
 
 int
 pk_hash_data (gnutls_pk_algorithm_t pk, gnutls_digest_algorithm_t hash,
-              bigint_t * params,
+              gnutls_pk_params_st* params,
               const gnutls_datum_t * data, gnutls_datum_t * digest)
 {
   int ret;
 
-  switch (pk)
-    {
-    case GNUTLS_PK_RSA:
-      break;
-    case GNUTLS_PK_DSA:
-      if (params && hash != _gnutls_dsa_q_to_hash (params[1]))
-        {
-          gnutls_assert ();
-          return GNUTLS_E_INVALID_REQUEST;
-        }
-      break;
-    default:
-      gnutls_assert ();
-      return GNUTLS_E_INVALID_REQUEST;
-    }
-
   digest->size = _gnutls_hash_get_algo_len (hash);
   digest->data = gnutls_malloc (digest->size);
   if (digest->data == NULL)
@@ -903,6 +885,7 @@ pk_prepare_hash (gnutls_pk_algorithm_t pk,
       _gnutls_free_datum (&old_digest);
       break;
     case GNUTLS_PK_DSA:
+    case GNUTLS_PK_ECC:
       break;
     default:
       gnutls_assert ();
