@@ -44,7 +44,7 @@
 #include <auth/cert.h>
 #include <auth/anon.h>
 #include <auth/psk.h>
-#include <gnutls_algorithms.h>
+#include <algorithms.h>
 #include <gnutls_rsa_export.h>
 #include <gnutls_extensions.h>
 #include <system.h>
@@ -65,6 +65,14 @@ _gnutls_session_cert_type_set (gnutls_session_t session,
   session->security_parameters.cert_type = ct;
 }
 
+void
+_gnutls_session_ecc_curve_set (gnutls_session_t session,
+                               gnutls_ecc_curve_t c)
+{
+  _gnutls_handshake_log("HSK[%p]: Selected ECC curve (%d)\n", session, c);
+  session->security_parameters.ecc_curve = c;
+}
+
 /**
  * gnutls_cipher_get:
  * @session: is a #gnutls_session_t structure.
@@ -443,6 +451,10 @@ gnutls_deinit (gnutls_session_t session)
 
   if (session->key != NULL)
     {
+      gnutls_pk_params_release(&session->key->ecdh_params);
+      _gnutls_mpi_release (&session->key->ecdh_x);
+      _gnutls_mpi_release (&session->key->ecdh_y);
+
       _gnutls_mpi_release (&session->key->KEY);
       _gnutls_mpi_release (&session->key->client_Y);
       _gnutls_mpi_release (&session->key->client_p);
@@ -1164,6 +1176,30 @@ _gnutls_session_is_psk (gnutls_session_t session)
   return 0;
 }
 
+/*-
+ * _gnutls_session_is_ecc - Used to check whether this session uses ECC kx
+ * @session: is a #gnutls_session_t structure.
+ *
+ * This function will return non zero if this session uses an elliptic
+ * curves key exchange exchange algorithm.
+ -*/
+int
+_gnutls_session_is_ecc (gnutls_session_t session)
+{
+  gnutls_kx_algorithm_t kx;
+
+  /* We get the key exchange algorithm through the ciphersuite because
+   * the negotiated key exchange might not have been set yet.
+   */
+  kx =
+    _gnutls_cipher_suite_get_kx_algo (&session->
+                                      security_parameters.current_cipher_suite);
+  if (kx == GNUTLS_KX_ECDHE_RSA || kx == GNUTLS_KX_ECDHE_ECDSA || kx == GNUTLS_KX_ANON_ECDH)
+    return 1;
+
+  return 0;
+}
+
 /**
  * gnutls_session_get_ptr:
  * @session: is a #gnutls_session_t structure.
