@@ -30,7 +30,7 @@
 #include "gnutls_errors.h"
 #include "gnutls_dh.h"
 #include "debug.h"
-#include "gnutls_algorithms.h"
+#include "algorithms.h"
 #include "gnutls_compress.h"
 #include "gnutls_cipher.h"
 #include "gnutls_buffers.h"
@@ -223,7 +223,7 @@ _gnutls_read_client_hello_v2 (gnutls_session_t session, opaque * data,
   _gnutls_tls_create_random (rnd);
   _gnutls_set_server_random (session, rnd);
 
-  session->security_parameters.timestamp = time (NULL);
+  session->security_parameters.timestamp = gnutls_time (NULL);
 
 
   /* RESUME SESSION */
