@@ -662,6 +662,12 @@ extern "C"
                                            const gnutls_datum_t * u,
                                            const gnutls_datum_t * e1,
                                            const gnutls_datum_t * e2);
+  int gnutls_x509_privkey_import_ecc_raw (gnutls_x509_privkey_t key,
+                                          gnutls_ecc_curve_t curve,
+                                          const gnutls_datum_t * x,
+                                          const gnutls_datum_t * y,
+                                          const gnutls_datum_t * k);
+
   int gnutls_x509_privkey_fix (gnutls_x509_privkey_t key);
 
   int gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
@@ -713,7 +719,10 @@ extern "C"
                                           gnutls_datum_t * p,
                                           gnutls_datum_t * q,
                                           gnutls_datum_t * u);
-
+  int gnutls_x509_privkey_export_ecc_raw (gnutls_x509_privkey_t key, 
+                                          gnutls_ecc_curve_t *curve,  
+                                          gnutls_datum_t * x, gnutls_datum_t * y,
+                                          gnutls_datum_t* k);
 /* Certificate request stuff.
  */
 
@@ -721,6 +730,8 @@ extern "C"
                              gnutls_certificate_print_formats_t format,
                              gnutls_datum_t * out);
 
+  int gnutls_x509_crq_verify (gnutls_x509_crq_t crq, unsigned int flags);
+
   int gnutls_x509_crq_init (gnutls_x509_crq_t * crq);
   void gnutls_x509_crq_deinit (gnutls_x509_crq_t crq);
   int gnutls_x509_crq_import (gnutls_x509_crq_t crq,
@@ -836,6 +847,9 @@ extern "C"
   void
   gnutls_x509_trust_list_deinit (gnutls_x509_trust_list_t list, unsigned int all);
 
+  int gnutls_trust_list_get_issuer(gnutls_x509_trust_list_t list,
+    gnutls_x509_crt_t cert, gnutls_x509_crt_t* issuer, unsigned int flags);
+
   int
   gnutls_x509_trust_list_add_cas (gnutls_x509_trust_list_t list, 
     const gnutls_x509_crt_t * clist, int clist_size, unsigned int flags);
