@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2010
- * Free Software Foundation, Inc.
+ * Copyright (C) 2010 Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -42,9 +41,12 @@
 #include <nettle/rsa.h>
 #include <random.h>
 #include <gnutls/crypto.h>
+#include "ecc.h"
 
 #define TOMPZ(x) (*((mpz_t*)(x)))
 
+static inline int is_supported_curve(int curve);
+
 static void
 rnd_func (void *_ctx, unsigned length, uint8_t * data)
 {
@@ -81,6 +83,84 @@ _rsa_params_to_privkey (const gnutls_pk_params_st * pk_params,
 
 }
 
+static void 
+_ecc_params_to_privkey(const gnutls_pk_params_st * pk_params,
+                       ecc_key * priv)
+{
+        priv->type = PK_PRIVATE;
+        memcpy(&priv->prime, pk_params->params[0], sizeof(mpz_t));
+        memcpy(&priv->order, pk_params->params[1], sizeof(mpz_t));
+        memcpy(&priv->A, pk_params->params[2], sizeof(mpz_t));
+        memcpy(&priv->Gx, pk_params->params[3], sizeof(mpz_t));
+        memcpy(&priv->Gy, pk_params->params[4], sizeof(mpz_t));
+        memcpy(&priv->pubkey.x, pk_params->params[5], sizeof(mpz_t));
+        memcpy(&priv->pubkey.y, pk_params->params[6], sizeof(mpz_t));
+        memcpy(&priv->k, pk_params->params[7], sizeof(mpz_t));
+        mpz_init_set_ui(priv->pubkey.z, 1);
+}
+
+static void 
+_ecc_params_to_pubkey(const gnutls_pk_params_st * pk_params,
+                       ecc_key * pub)
+{
+        pub->type = PK_PUBLIC;
+        memcpy(&pub->prime, pk_params->params[0], sizeof(mpz_t));
+        memcpy(&pub->order, pk_params->params[1], sizeof(mpz_t));
+        memcpy(&pub->A, pk_params->params[2], sizeof(mpz_t));
+        memcpy(&pub->Gx, pk_params->params[3], sizeof(mpz_t));
+        memcpy(&pub->Gy, pk_params->params[4], sizeof(mpz_t));
+        memcpy(&pub->pubkey.x, pk_params->params[5], sizeof(mpz_t));
+        memcpy(&pub->pubkey.y, pk_params->params[6], sizeof(mpz_t));
+        mpz_init_set_ui(pub->pubkey.z, 1);
+}
+
+static int _wrap_nettle_pk_derive(gnutls_pk_algorithm_t algo, gnutls_datum_t * out,
+                                  const gnutls_pk_params_st * priv,
+                                  const gnutls_pk_params_st * pub)
+{
+  int ret;
+
+  switch (algo)
+    {
+    case GNUTLS_PK_ECC:
+      {
+        ecc_key ecc_pub, ecc_priv;
+        int curve = priv->flags;
+        unsigned long sz;
+
+        if (is_supported_curve(curve) == 0)
+          return gnutls_assert_val(GNUTLS_E_ECC_NO_SUPPORTED_CURVES);
+
+        _ecc_params_to_pubkey(pub, &ecc_pub);
+        _ecc_params_to_privkey(priv, &ecc_priv);
+
+        sz = ECC_BUF_SIZE;
+        out->data = gnutls_malloc(sz);
+        if (out->data == NULL)
+          return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+        ret = ecc_shared_secret(&ecc_priv, &ecc_pub, out->data, &sz);
+        if (ret != 0)
+          {
+            gnutls_free(out->data);
+            return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+          }
+        out->size = sz;
+        break;
+      }
+    default:
+      gnutls_assert ();
+      ret = GNUTLS_E_INTERNAL_ERROR;
+      goto cleanup;
+    }
+
+  ret = 0;
+
+cleanup:
+
+  return ret;
+}
+
 static int
 _wrap_nettle_pk_encrypt (gnutls_pk_algorithm_t algo,
                          gnutls_datum_t * ciphertext,
@@ -89,7 +169,6 @@ _wrap_nettle_pk_encrypt (gnutls_pk_algorithm_t algo,
 {
   int ret;
 
-  /* make a sexp from pkey */
   switch (algo)
     {
     case GNUTLS_PK_RSA:
@@ -270,7 +349,45 @@ _wrap_nettle_pk_sign (gnutls_pk_algorithm_t algo,
 
   switch (algo)
     {
+    case GNUTLS_PK_ECC: /* we do ECDSA */
+      {
+        ecc_key priv;
+        struct dsa_signature sig;
+        int hash_len;
 
+        _ecc_params_to_privkey(pk_params, &priv);
+
+        dsa_signature_init (&sig);
+
+        hash = _gnutls_dsa_q_to_hash (algo, pk_params);
+        hash_len = _gnutls_hash_get_algo_len (hash);
+        if (hash_len > vdata->size)
+          {
+            gnutls_assert ();
+            _gnutls_debug_log("Security level of algorithm requires hash %s or better\n", gnutls_mac_get_name(hash));
+          }
+
+        ret = ecc_sign_hash(vdata->data, vdata->size, 
+                            &sig, NULL, rnd_func, &priv);
+        if (ret != 0)
+          {
+            gnutls_assert ();
+            ret = GNUTLS_E_PK_SIGN_FAILED;
+            goto ecdsa_fail;
+          }
+
+        ret = _gnutls_encode_ber_rs (signature, &sig.r, &sig.s);
+
+      ecdsa_fail:
+        dsa_signature_clear (&sig);
+
+        if (ret < 0)
+          {
+            gnutls_assert ();
+            goto cleanup;
+          }
+        break;
+      }
     case GNUTLS_PK_DSA:
       {
         struct dsa_public_key pub;
@@ -285,14 +402,12 @@ _wrap_nettle_pk_sign (gnutls_pk_algorithm_t algo,
 
         dsa_signature_init (&sig);
 
-        hash = _gnutls_dsa_q_to_hash (pub.q);
+        hash = _gnutls_dsa_q_to_hash (algo, pk_params);
         hash_len = _gnutls_hash_get_algo_len (hash);
         if (hash_len > vdata->size)
           {
             gnutls_assert ();
-            _gnutls_debug_log("Asked to sign %d bytes with hash %s\n", vdata->size, gnutls_mac_get_name(hash));
-            ret = GNUTLS_E_PK_SIGN_FAILED;
-            goto dsa_fail;
+            _gnutls_debug_log("Security level of algorithm requires hash %s or better\n", gnutls_mac_get_name(hash));
           }
 
         ret =
@@ -414,6 +529,50 @@ _wrap_nettle_pk_verify (gnutls_pk_algorithm_t algo,
 
   switch (algo)
     {
+    case GNUTLS_PK_ECC: /* ECDSA */
+      {
+        ecc_key pub;
+        struct dsa_signature sig;
+        int stat;
+
+        ret = _gnutls_decode_ber_rs (signature, &tmp[0], &tmp[1]);
+        if (ret < 0)
+          {
+            gnutls_assert ();
+            goto cleanup;
+          }
+
+        _ecc_params_to_pubkey(pk_params, &pub);
+        memcpy (&sig.r, tmp[0], sizeof (sig.r));
+        memcpy (&sig.s, tmp[1], sizeof (sig.s));
+
+        hash = _gnutls_dsa_q_to_hash (algo, pk_params);
+        if (vdata->size != _gnutls_hash_get_algo_len (hash))
+          {
+            gnutls_assert ();
+            
+            if (vdata->size != 20)
+              {
+                ret = GNUTLS_E_PK_SIG_VERIFY_FAILED;
+                goto ecdsa_fail;
+              }
+            else hash = GNUTLS_DIG_SHA1;
+          }
+
+        ret = ecc_verify_hash(&sig, vdata->data, vdata->size, &stat, &pub);
+        if (ret != 0 || stat != 1)
+          {
+            gnutls_assert();
+            ret = GNUTLS_E_PK_SIG_VERIFY_FAILED;
+          }
+        else
+          ret = 0;
+
+      ecdsa_fail:
+        _gnutls_mpi_release (&tmp[0]);
+        _gnutls_mpi_release (&tmp[1]);
+        break;
+      }
     case GNUTLS_PK_DSA:
       {
         struct dsa_public_key pub;
@@ -430,13 +589,17 @@ _wrap_nettle_pk_verify (gnutls_pk_algorithm_t algo,
         memcpy (&sig.r, tmp[0], sizeof (sig.r));
         memcpy (&sig.s, tmp[1], sizeof (sig.s));
 
-        hash = _gnutls_dsa_q_to_hash (pub.q);
-
+        hash = _gnutls_dsa_q_to_hash (algo, pk_params);
         if (vdata->size != _gnutls_hash_get_algo_len (hash))
           {
             gnutls_assert ();
-            ret = GNUTLS_E_PK_SIG_VERIFY_FAILED;
-            goto dsa_fail;
+            
+            if (vdata->size != 20)
+              {
+                ret = GNUTLS_E_PK_SIG_VERIFY_FAILED;
+                goto dsa_fail;
+              }
+            else hash = GNUTLS_DIG_SHA1;
           }
 
         ret = _dsa_verify (&pub, vdata->size, vdata->data, &sig);
@@ -486,6 +649,15 @@ cleanup:
   return ret;
 }
 
+static inline int is_supported_curve(int curve)
+{
+  if (gnutls_ecc_curve_get_name(curve) != NULL)
+    return 1;
+  else
+    return 0;
+}
+
+
 static int
 wrap_nettle_pk_generate_params (gnutls_pk_algorithm_t algo,
                                 unsigned int level /*bits */ ,
@@ -600,6 +772,57 @@ rsa_fail:
         rsa_private_key_clear (&priv);
         rsa_public_key_clear (&pub);
 
+        if (ret < 0)
+          goto fail;
+
+        break;
+      }
+    case GNUTLS_PK_ECC:
+      {
+        ecc_key key;
+        ecc_set_type tls_ecc_set;
+        const gnutls_ecc_curve_entry_st *st;
+
+        st = _gnutls_ecc_curve_get_params(level);
+        if (st == NULL)
+          return gnutls_assert_val(GNUTLS_E_ECC_NO_SUPPORTED_CURVES);
+        
+        tls_ecc_set.size = st->size;
+        tls_ecc_set.prime = st->prime;
+        tls_ecc_set.order = st->order;
+        tls_ecc_set.Gx = st->Gx;
+        tls_ecc_set.Gy = st->Gy;
+        tls_ecc_set.A = st->A;
+
+        ret = ecc_make_key(NULL, rnd_func, &key, &tls_ecc_set);
+        if (ret != 0)
+          return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+        params->params_nr = 0;
+        for (i = 0; i < ECC_PRIVATE_PARAMS; i++)
+          {
+            params->params[i] = _gnutls_mpi_alloc_like(&key.prime);
+            if (params->params[i] == NULL)
+              {
+                ret = GNUTLS_E_MEMORY_ERROR;
+                goto ecc_fail;
+              }
+            params->params_nr++;
+          }
+        params->flags = level;
+
+        mpz_set(TOMPZ(params->params[0]), key.prime);
+        mpz_set(TOMPZ(params->params[1]), key.order);
+        mpz_set(TOMPZ(params->params[2]), key.A);
+        mpz_set(TOMPZ(params->params[3]), key.Gx);
+        mpz_set(TOMPZ(params->params[4]), key.Gy);
+        mpz_set(TOMPZ(params->params[5]), key.pubkey.x);
+        mpz_set(TOMPZ(params->params[6]), key.pubkey.y);
+        mpz_set(TOMPZ(params->params[7]), key.k);
+        
+ecc_fail:
+        ecc_free(&key);
+        
         if (ret < 0)
           goto fail;
 
@@ -645,7 +868,7 @@ wrap_nettle_pk_fixup (gnutls_pk_algorithm_t algo,
       _gnutls_mpi_release (&params->params[6]);
       _gnutls_mpi_release (&params->params[7]);
 
-      result = _gnutls_calc_rsa_exp (params->params, RSA_PRIVATE_PARAMS - 2);
+      result = _gnutls_calc_rsa_exp (params);
       if (result < 0)
         {
           gnutls_assert ();
@@ -666,4 +889,5 @@ gnutls_crypto_pk_st _gnutls_pk_ops = {
   .verify = _wrap_nettle_pk_verify,
   .generate = wrap_nettle_pk_generate_params,
   .pk_fixup_private_params = wrap_nettle_pk_fixup,
+  .derive = _wrap_nettle_pk_derive,
 };
