@@ -27,8 +27,8 @@
 
 #include "types.h"
 
-#define _cdk_log_debug _gnutls_debug_log
-#define _cdk_log_info _gnutls_x509_log
+#define _cdk_log_debug _gnutls_hard_log
+#define _cdk_log_info _gnutls_debug_log
 #define _cdk_get_log_level() _gnutls_log_level
 
 #define cdk_malloc gnutls_malloc
@@ -106,6 +106,10 @@ FILE *_cdk_tmpfile (void);
     _cdk_memistr((haystack), strlen (haystack), (needle))
 
 /*-- proc-packet.c --*/
+cdk_error_t _cdk_proc_packets (cdk_ctx_t hd, cdk_stream_t inp,
+                               cdk_stream_t data,
+                               const char *output, cdk_stream_t outstream,
+                               digest_hd_st * md);
 cdk_error_t _cdk_pkt_write2 (cdk_stream_t out, int pkttype, void *pktctx);
 
 /*-- pubkey.c --*/
