@@ -343,7 +343,7 @@ pkcs11_get_info (struct pkcs11_url_info *info,
       return GNUTLS_E_SHORT_MEMORY_BUFFER;
     }
 
-  strcpy (output, str);
+  memcpy (output, str, len + 1);
 
   *output_size = len;
 
@@ -355,7 +355,7 @@ static int init = 0;
 
 /**
  * gnutls_pkcs11_init:
- * @flags: GNUTLS_PKCS11_FLAG_MANUAL or GNUTLS_PKCS11_FLAG_AUTO
+ * @flags: %GNUTLS_PKCS11_FLAG_MANUAL or %GNUTLS_PKCS11_FLAG_AUTO
  * @configfile: either NULL or the location of a configuration file
  *
  * This function will initialize the PKCS 11 subsystem in gnutls. It will
@@ -364,7 +364,8 @@ static int init = 0;
  * if %GNUTLS_PKCS11_FLAG_MANUAL is specified.
  *
  * Normally you don't need to call this function since it is being called
- * by gnutls_global_init(). Otherwise you must call it before it.
+ * by gnutls_global_init() using the %GNUTLS_PKCS11_FLAG_AUTO. If other option
+ * is required then it must be called before it.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  *   negative error value.
@@ -426,6 +427,7 @@ gnutls_pkcs11_init (unsigned int flags, const char *configfile)
                 }
             }
         }
+      fclose(fp);
     }
 
   return 0;
@@ -676,7 +678,7 @@ pkcs11_url_to_info (const char *url, struct pkcs11_url_info *info)
   if ((p1 = strstr (url, "objecttype=")) != NULL)
     {
       p1 += sizeof ("objecttype=") - 1;
-      l = sizeof (info->model);
+      l = sizeof (info->type);
 
       ret = unescape_string (info->type, p1, &l, ';');
       if (ret < 0)
@@ -689,33 +691,21 @@ pkcs11_url_to_info (const char *url, struct pkcs11_url_info *info)
       || ((p1 = strstr (url, ":id=")) != NULL))
     {
       p1 += sizeof (";id=") - 1;
+      l = sizeof (info->certid_raw);
 
-      if ((p2 = strchr (p1, ';')) == NULL)
-        {
-          l = strlen (p1);
-        }
-      else
-        {
-          l = p2 - p1;
-        }
-
-      if (l > sizeof (info->id) - 1)
-        {
-          gnutls_assert ();
-          ret = GNUTLS_E_PARSING_ERROR;
-        }
-
-      memcpy (info->id, p1, l);
-      info->id[l] = 0;
-
-      /* convert to raw */
-      info->certid_raw_size = sizeof (info->certid_raw);
-      ret =
-        _gnutls_hex2bin (info->id, strlen (info->id),
-                         info->certid_raw, &info->certid_raw_size);
+      ret = unescape_string (info->certid_raw, p1, &l, ';');
       if (ret < 0)
         {
-          gnutls_assert ();
+          goto cleanup;
+        }
+      /* not null terminated */
+      info->certid_raw_size = l-1;
+
+      p2 = _gnutls_bin2hex(info->certid_raw, info->certid_raw_size,
+                           info->id, sizeof(info->id), ":");
+      if (p2 == NULL)
+        {
+          ret = GNUTLS_E_PARSING_ERROR;
           goto cleanup;
         }
     }
@@ -730,21 +720,24 @@ cleanup:
 
 #define INVALID_CHARS       "\\/\"'%&#@!?$* <>{}[]()`|:;,.+-"
 
+/* Appends @tname to @dest under the name @p11name.
+ * init indicates whether it is the initial addition to buffer.
+ */
 static int
-append (gnutls_buffer_st * dest, const char *tname,
-        const char *p11name, int init)
+append (gnutls_buffer_st * dest, const void *tname, int tname_size,
+        const char *p11name, int all, int init)
 {
   gnutls_buffer_st tmpstr;
   int ret;
 
   _gnutls_buffer_init (&tmpstr);
-  if ((ret = _gnutls_buffer_append_str (&tmpstr, tname)) < 0)
+  if ((ret = _gnutls_buffer_append_data (&tmpstr, tname, tname_size)) < 0)
     {
       gnutls_assert ();
       goto cleanup;
     }
 
-  ret = _gnutls_buffer_escape (&tmpstr, INVALID_CHARS);
+  ret = _gnutls_buffer_escape (&tmpstr, all, INVALID_CHARS);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -790,7 +783,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
   if (info->token[0])
     {
-      ret = append (&str, info->token, "token", init);
+      ret = append (&str, info->token, strlen(info->token), "token", 0, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -801,7 +794,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
   if (info->serial[0])
     {
-      ret = append (&str, info->serial, "serial", init);
+      ret = append (&str, info->serial, strlen(info->serial), "serial", 0, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -812,7 +805,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
   if (info->model[0])
     {
-      ret = append (&str, info->model, "model", init);
+      ret = append (&str, info->model, strlen(info->model), "model", 0, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -824,7 +817,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
   if (info->manufacturer[0])
     {
-      ret = append (&str, info->manufacturer, "manufacturer", init);
+      ret = append (&str, info->manufacturer, strlen(info->manufacturer), "manufacturer", 0, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -835,7 +828,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
   if (info->label[0])
     {
-      ret = append (&str, info->label, "object", init);
+      ret = append (&str, info->label, strlen(info->label), "object", 0, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -846,7 +839,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
   if (info->type[0])
     {
-      ret = append (&str, info->type, "objecttype", init);
+      ret = append (&str, info->type, strlen(info->type), "objecttype", 0, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -860,8 +853,8 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
       if (info->lib_manufacturer[0])
         {
           ret =
-            append (&str, info->lib_manufacturer, "library-manufacturer",
-                    init);
+            append (&str, info->lib_manufacturer, strlen(info->lib_manufacturer), "library-manufacturer",
+                    0, init);
           if (ret < 0)
             {
               gnutls_assert ();
@@ -872,7 +865,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
 
       if (info->lib_desc[0])
         {
-          ret = append (&str, info->lib_desc, "library-description", init);
+          ret = append (&str, info->lib_desc, strlen(info->lib_desc), "library-description", 0, init);
           if (ret < 0)
             {
               gnutls_assert ();
@@ -886,7 +879,7 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
     {
       if (info->lib_version[0])
         {
-          ret = append (&str, info->lib_version, "library-version", init);
+          ret = append (&str, info->lib_version, strlen(info->lib_version), "library-version", 0, init);
           if (ret < 0)
             {
               gnutls_assert ();
@@ -896,9 +889,9 @@ pkcs11_info_to_url (const struct pkcs11_url_info *info,
         }
     }
 
-  if (info->id[0] != 0)
+  if (info->certid_raw_size > 0)
     {
-      ret = _gnutls_buffer_append_printf (&str, ";id=%s", info->id);
+      ret = append (&str, info->certid_raw, info->certid_raw_size, "id", 1, init);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -1333,7 +1326,7 @@ pkcs11_obj_import (unsigned int class, gnutls_pkcs11_obj_t obj,
     }
 
   if (obj->type != GNUTLS_PKCS11_OBJ_UNKNOWN)
-    strcpy (obj->info.type, pkcs11_obj_type_to_str (obj->type));
+    _gnutls_str_cpy (obj->info.type, sizeof(obj->info.type), pkcs11_obj_type_to_str (obj->type));
 
   if (data && data->data)
     {
@@ -1856,13 +1849,13 @@ find_token_num (pakchois_session_t * pks,
 
   if (find_data->current == find_data->seq)
     {
-      strcpy (find_data->info.manufacturer, tinfo->tinfo.manufacturer_id);
-      strcpy (find_data->info.token, tinfo->tinfo.label);
-      strcpy (find_data->info.model, tinfo->tinfo.model);
-      strcpy (find_data->info.serial, tinfo->tinfo.serial_number);
+      _gnutls_str_cpy (find_data->info.manufacturer, sizeof(find_data->info.manufacturer), tinfo->tinfo.manufacturer_id);
+      _gnutls_str_cpy (find_data->info.token, sizeof(find_data->info.token), tinfo->tinfo.label);
+      _gnutls_str_cpy (find_data->info.model, sizeof(find_data->info.model), tinfo->tinfo.model);
+      _gnutls_str_cpy (find_data->info.serial, sizeof(find_data->info.serial), tinfo->tinfo.serial_number);
 
-      strcpy (find_data->info.lib_manufacturer, lib_info->manufacturer_id);
-      strcpy (find_data->info.lib_desc, lib_info->library_description);
+      _gnutls_str_cpy (find_data->info.lib_manufacturer, sizeof(find_data->info.lib_manufacturer), lib_info->manufacturer_id);
+      _gnutls_str_cpy (find_data->info.lib_desc, sizeof(find_data->info.lib_desc), lib_info->library_description);
       snprintf (find_data->info.lib_version,
                 sizeof (find_data->info.lib_version), "%u.%u",
                 (unsigned int) lib_info->library_version.major,
@@ -1975,7 +1968,7 @@ gnutls_pkcs11_token_get_info (const char *url,
       return GNUTLS_E_SHORT_MEMORY_BUFFER;
     }
 
-  strcpy (output, str);
+  memcpy (output, str, len+1);
 
   *output_size = len;
 
@@ -2048,10 +2041,10 @@ pkcs11_login (pakchois_session_t * pks, const struct token_info *info, int so)
     }
 
   memset (&uinfo, 0, sizeof (uinfo));
-  strcpy (uinfo.manufacturer, info->tinfo.manufacturer_id);
-  strcpy (uinfo.token, info->tinfo.label);
-  strcpy (uinfo.model, info->tinfo.model);
-  strcpy (uinfo.serial, info->tinfo.serial_number);
+  _gnutls_str_cpy (uinfo.manufacturer, sizeof(uinfo.manufacturer), info->tinfo.manufacturer_id);
+  _gnutls_str_cpy (uinfo.token, sizeof(uinfo.token), info->tinfo.label);
+  _gnutls_str_cpy (uinfo.model, sizeof(uinfo.model), info->tinfo.model);
+  _gnutls_str_cpy (uinfo.serial, sizeof(uinfo.serial), info->tinfo.serial_number);
   ret = pkcs11_info_to_url (&uinfo, 1, &token_url);
   if (ret < 0)
     {
