@@ -26,7 +26,8 @@
 #ifndef COMMON_H
 #define COMMON_H
 
-#include <gnutls_algorithms.h>
+#include <algorithms.h>
+#include <abstract_int.h>
 #include <x509/x509_int.h>
 
 #define MAX_STRING_LEN 512
@@ -130,8 +131,7 @@ int _gnutls_x509_get_pk_algorithm (ASN1_TYPE src, const char *src_name,
 int _gnutls_x509_encode_and_copy_PKI_params (ASN1_TYPE dst,
                                              const char *dst_name,
                                              gnutls_pk_algorithm_t
-                                             pk_algorithm, bigint_t * params,
-                                             int params_size);
+                                             pk_algorithm, gnutls_pk_params_st * params);
 int _gnutls_asn1_copy_node (ASN1_TYPE * dst, const char *dst_name,
                             ASN1_TYPE src, const char *src_name);
 
@@ -140,13 +140,13 @@ int _gnutls_x509_get_signed_data (ASN1_TYPE src, const char *src_name,
 int _gnutls_x509_get_signature (ASN1_TYPE src, const char *src_name,
                                 gnutls_datum_t * signature);
 
-gnutls_digest_algorithm_t _gnutls_dsa_q_to_hash (bigint_t q);
+gnutls_digest_algorithm_t _gnutls_dsa_q_to_hash (gnutls_pk_algorithm_t algo, const gnutls_pk_params_st* params);
 
 int _gnutls_get_asn_mpis (ASN1_TYPE asn, const char *root,
-                          bigint_t * params, int *params_size);
+                          gnutls_pk_params_st * params);
 
-int _gnutls_get_key_id (gnutls_pk_algorithm_t pk, bigint_t * params,
-                        int params_size, unsigned char *output_data,
+int _gnutls_get_key_id (gnutls_pk_algorithm_t pk, gnutls_pk_params_st*,
+                        unsigned char *output_data,
                         size_t * output_data_size);
 
 void _asnstr_append_name (char *name, size_t name_size, const char *part1,
@@ -155,8 +155,8 @@ void _asnstr_append_name (char *name, size_t name_size, const char *part1,
 int pubkey_verify_sig (const gnutls_datum_t * tbs,
                        const gnutls_datum_t * hash,
                        const gnutls_datum_t * signature,
-                       gnutls_pk_algorithm_t pk, bigint_t * issuer_params,
-                       int issuer_params_size);
+                       gnutls_pk_algorithm_t pk, 
+                       gnutls_pk_params_st * issuer_params);
 
 int
 check_if_same_cert (gnutls_x509_crt_t cert1, gnutls_x509_crt_t cert2);
