@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free
- * Software Foundation, Inc.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+ * Free Software Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -306,7 +306,7 @@ gnutls_x509_crt_t issuer = NULL;
             }
           else
             {
-              time_t now = time(0);
+              time_t now = gnutls_time(0);
 
               if (now < gnutls_x509_crt_get_expiration_time(trusted_cas[i]) && 
                 now >= gnutls_x509_crt_get_activation_time(trusted_cas[i]))
@@ -549,7 +549,7 @@ _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
 {
   int i = 0, ret;
   unsigned int status = 0, output;
-  time_t now = time (0);
+  time_t now = gnutls_time (0);
   gnutls_x509_crt_t issuer = NULL;
 
   if (clist_size > 1)
@@ -752,8 +752,8 @@ decode_ber_digest_info (const gnutls_datum_t * info,
 static int
 _pkcs1_rsa_verify_sig (const gnutls_datum_t * text,
                        const gnutls_datum_t * prehash,
-                       const gnutls_datum_t * signature, bigint_t * params,
-                       int params_len)
+                       const gnutls_datum_t * signature, 
+                       gnutls_pk_params_st * params)
 {
   gnutls_mac_algorithm_t hash = GNUTLS_MAC_UNKNOWN;
   int ret;
@@ -763,7 +763,7 @@ _pkcs1_rsa_verify_sig (const gnutls_datum_t * text,
   gnutls_datum_t decrypted;
 
   ret =
-    _gnutls_pkcs1_rsa_decrypt (&decrypted, signature, params, params_len, 1);
+    _gnutls_pkcs1_rsa_decrypt (&decrypted, signature, params, 1);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -824,13 +824,14 @@ _pkcs1_rsa_verify_sig (const gnutls_datum_t * text,
   return 0;
 }
 
-/* Hashes input data and verifies a DSA signature.
+/* Hashes input data and verifies a signature.
  */
 static int
-dsa_verify_sig (const gnutls_datum_t * text,
+verify_sig (const gnutls_datum_t * text,
                 const gnutls_datum_t * hash,
-                const gnutls_datum_t * signature, bigint_t * params,
-                int params_len)
+                const gnutls_datum_t * signature,
+                gnutls_pk_algorithm_t pk,
+                gnutls_pk_params_st* params)
 {
   int ret;
   opaque _digest[MAX_HASH_SIZE];
@@ -838,21 +839,24 @@ dsa_verify_sig (const gnutls_datum_t * text,
   digest_hd_st hd;
   gnutls_digest_algorithm_t algo;
 
-  algo = _gnutls_dsa_q_to_hash (params[1]);
+  algo = _gnutls_dsa_q_to_hash (pk, params);
   if (hash)
     {
       /* SHA1 or better allowed */
       if (!hash->data || hash->size != _gnutls_hash_get_algo_len(algo))
         {
           gnutls_assert();
-          _gnutls_debug_log("Hash size (%d) does not correspond to hash %s", (int)hash->size, gnutls_mac_get_name(algo));
-          return GNUTLS_E_INVALID_REQUEST;
+          _gnutls_debug_log("Hash size (%d) does not correspond to hash %s. Assuming SHA-1.\n", (int)hash->size, gnutls_mac_get_name(algo));
+          algo = GNUTLS_DIG_SHA1;
+          
+          if (hash->size != 20)
+            return gnutls_assert_val(GNUTLS_E_PK_SIG_VERIFY_FAILED);
         }
+
       digest = *hash;
     }
   else
     {
-
       ret = _gnutls_hash_init (&hd, algo);
       if (ret < 0)
         {
@@ -867,7 +871,7 @@ dsa_verify_sig (const gnutls_datum_t * text,
       digest.size = _gnutls_hash_get_algo_len(algo);
     }
 
-  ret = _gnutls_dsa_verify (&digest, signature, params, params_len);
+  ret = _gnutls_pk_verify (pk, &digest, signature, params);
 
   return ret;
 }
@@ -879,8 +883,8 @@ int
 pubkey_verify_sig (const gnutls_datum_t * tbs,
                    const gnutls_datum_t * hash,
                    const gnutls_datum_t * signature,
-                   gnutls_pk_algorithm_t pk, bigint_t * issuer_params,
-                   int issuer_params_size)
+                   gnutls_pk_algorithm_t pk,
+                   gnutls_pk_params_st * issuer_params)
 {
 
   switch (pk)
@@ -888,7 +892,7 @@ pubkey_verify_sig (const gnutls_datum_t * tbs,
     case GNUTLS_PK_RSA:
 
       if (_pkcs1_rsa_verify_sig
-          (tbs, hash, signature, issuer_params, issuer_params_size) != 0)
+          (tbs, hash, signature, issuer_params) != 0)
         {
           gnutls_assert ();
           return GNUTLS_E_PK_SIG_VERIFY_FAILED;
@@ -897,9 +901,9 @@ pubkey_verify_sig (const gnutls_datum_t * tbs,
       return 1;
       break;
 
+    case GNUTLS_PK_ECC:
     case GNUTLS_PK_DSA:
-      if (dsa_verify_sig
-          (tbs, hash, signature, issuer_params, issuer_params_size) != 0)
+      if (verify_sig(tbs, hash, signature, pk, issuer_params) != 0)
         {
           gnutls_assert ();
           return GNUTLS_E_PK_SIG_VERIFY_FAILED;
@@ -915,9 +919,14 @@ pubkey_verify_sig (const gnutls_datum_t * tbs,
 }
 
 gnutls_digest_algorithm_t
-_gnutls_dsa_q_to_hash (bigint_t q)
+_gnutls_dsa_q_to_hash (gnutls_pk_algorithm_t algo, const gnutls_pk_params_st* params)
 {
-  int bits = _gnutls_mpi_get_nbits (q);
+  int bits = 0;
+  
+  if (algo == GNUTLS_PK_DSA)
+    bits = _gnutls_mpi_get_nbits (params->params[1]);
+  else if (algo == GNUTLS_PK_ECC)
+    bits = gnutls_ecc_curve_get_size(params->flags)*8;
 
   if (bits <= 160)
     {
@@ -941,8 +950,7 @@ int
 _gnutls_x509_verify_algorithm (gnutls_mac_algorithm_t * hash,
                                const gnutls_datum_t * signature,
                                gnutls_pk_algorithm_t pk,
-                               bigint_t * issuer_params,
-                               unsigned int issuer_params_size)
+                               gnutls_pk_params_st * issuer_params)
 {
   opaque digest[MAX_HASH_SIZE];
   gnutls_datum_t decrypted;
@@ -952,9 +960,10 @@ _gnutls_x509_verify_algorithm (gnutls_mac_algorithm_t * hash,
   switch (pk)
     {
     case GNUTLS_PK_DSA:
+    case GNUTLS_PK_ECC:
 
       if (hash)
-        *hash = _gnutls_dsa_q_to_hash (issuer_params[1]);
+        *hash = _gnutls_dsa_q_to_hash (pk, issuer_params);
 
       ret = 0;
       break;
@@ -968,7 +977,7 @@ _gnutls_x509_verify_algorithm (gnutls_mac_algorithm_t * hash,
 
       ret =
         _gnutls_pkcs1_rsa_decrypt (&decrypted, signature,
-                                   issuer_params, issuer_params_size, 1);
+                                   issuer_params, 1);
 
 
       if (ret < 0)
@@ -1021,14 +1030,13 @@ _gnutls_x509_verify_signature (const gnutls_datum_t * tbs,
                                const gnutls_datum_t * signature,
                                gnutls_x509_crt_t issuer)
 {
-  bigint_t issuer_params[MAX_PUBLIC_PARAMS_SIZE];
-  int ret, issuer_params_size, i;
+  gnutls_pk_params_st issuer_params;
+  int ret;
 
   /* Read the MPI parameters from the issuer's certificate.
    */
-  issuer_params_size = MAX_PUBLIC_PARAMS_SIZE;
   ret =
-    _gnutls_x509_crt_get_mpis (issuer, issuer_params, &issuer_params_size);
+    _gnutls_x509_crt_get_mpis (issuer, &issuer_params);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1038,7 +1046,7 @@ _gnutls_x509_verify_signature (const gnutls_datum_t * tbs,
   ret =
     pubkey_verify_sig (tbs, hash, signature,
                        gnutls_x509_crt_get_pk_algorithm (issuer, NULL),
-                       issuer_params, issuer_params_size);
+                       &issuer_params);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1046,10 +1054,7 @@ _gnutls_x509_verify_signature (const gnutls_datum_t * tbs,
 
   /* release all allocated MPIs
    */
-  for (i = 0; i < issuer_params_size; i++)
-    {
-      _gnutls_mpi_release (&issuer_params[i]);
-    }
+  gnutls_pk_params_release(&issuer_params);
 
   return ret;
 }
@@ -1068,7 +1073,7 @@ _gnutls_x509_privkey_verify_signature (const gnutls_datum_t * tbs,
   int ret;
 
   ret = pubkey_verify_sig (tbs, NULL, signature, issuer->pk_algorithm,
-                           issuer->params, issuer->params_size);
+                           &issuer->params);
   if (ret < 0)
     {
       gnutls_assert ();
