@@ -22,7 +22,6 @@
 #include <config.h>
 
 #include <gnutls/gnutls.h>
-#include <gnutls/extra.h>
 #include <gnutls/x509.h>
 #include <gnutls/openpgp.h>
 #include <gnutls/pkcs12.h>
@@ -47,9 +46,13 @@
 #include <version-etc.h>
 
 #include <certtool-cfg.h>
+#include <p11common.h>
 #include "certtool-gaa.h"
 #include "certtool-common.h"
 
+#define SIGN_HASH GNUTLS_DIG_SHA256
+
+static void privkey_info_int (gnutls_x509_privkey_t key);
 static void print_crl_info (gnutls_x509_crl_t crl, FILE * out);
 void pkcs7_info (void);
 void crq_info (void);
@@ -141,6 +144,21 @@ print_dsa_pkey (gnutls_datum_t * x, gnutls_datum_t * y, gnutls_datum_t * p,
   print_hex_datum (g);
 }
 
+static void
+print_ecc_pkey (gnutls_ecc_curve_t curve, gnutls_datum_t* k, gnutls_datum_t * x, gnutls_datum_t * y)
+{
+  fprintf (outfile, "curve:\t%s\n", gnutls_ecc_curve_get_name(curve));
+  if (k)
+    {
+      fprintf (outfile, "private key:");
+      print_hex_datum (k);
+    }
+  fprintf (outfile, "x:");
+  print_hex_datum (x);
+  fprintf (outfile, "y:");
+  print_hex_datum (y);
+}
+
 static void
 print_rsa_pkey (gnutls_datum_t * m, gnutls_datum_t * e, gnutls_datum_t * d,
                 gnutls_datum_t * p, gnutls_datum_t * q, gnutls_datum_t * u,
@@ -238,9 +256,9 @@ generate_private_key_int (void)
   int ret, key_type, bits;
 
   if (info.dsa)
-    {
-      key_type = GNUTLS_PK_DSA;
-    }
+    key_type = GNUTLS_PK_DSA;
+  else if (info.ecc)
+    key_type = GNUTLS_PK_ECC;
   else
     key_type = GNUTLS_PK_RSA;
 
@@ -262,7 +280,7 @@ generate_private_key_int (void)
     fprintf (stderr,
              "Note that DSA keys with size over 1024 can only be used with TLS 1.2 or later.\n\n");
 
-  ret = gnutls_x509_privkey_generate (key, key_type,bits, 0);
+  ret = gnutls_x509_privkey_generate (key, key_type, bits, 0);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "privkey_generate: %s", gnutls_strerror (ret));
 
@@ -315,6 +333,8 @@ print_private_key (gnutls_x509_privkey_t key)
   if (!key)
     return;
 
+  privkey_info_int(key);
+
   if (!info.pkcs8)
     {
       size = buffer_size;
@@ -362,12 +382,13 @@ generate_private_key (void)
 
 
 static gnutls_x509_crt_t
-generate_certificate (gnutls_x509_privkey_t * ret_key,
+generate_certificate (gnutls_privkey_t * ret_key,
                       gnutls_x509_crt_t ca_crt, int proxy,
                       common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt;
-  gnutls_x509_privkey_t key = NULL;
+  gnutls_privkey_t key = NULL;
+  gnutls_pubkey_t pubkey;
   size_t size;
   int ret;
   int client;
@@ -386,6 +407,8 @@ generate_certificate (gnutls_x509_privkey_t * ret_key,
     {
 
       key = load_private_key (1, cinfo);
+      
+      pubkey = load_public_key_or_import (1, key, cinfo);
 
       if (!batch)
         fprintf (stderr,
@@ -422,7 +445,7 @@ generate_certificate (gnutls_x509_privkey_t * ret_key,
           get_pkcs9_email_crt_set (crt);
         }
 
-      result = gnutls_x509_crt_set_key (crt, key);
+      result = gnutls_x509_crt_set_pubkey (crt, pubkey);
       if (result < 0)
         error (EXIT_FAILURE, 0, "set_key: %s", gnutls_strerror (result));
     }
@@ -803,7 +826,7 @@ void
 generate_self_signed (common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt;
-  gnutls_x509_privkey_t key;
+  gnutls_privkey_t key;
   size_t size;
   int result;
   const char *uri;
@@ -830,7 +853,7 @@ generate_self_signed (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_sign2 (crt, crt, key, get_dig (crt), 0);
+  result = gnutls_x509_crt_privkey_sign (crt, crt, key, get_dig (crt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -842,17 +865,17 @@ generate_self_signed (common_info_st * cinfo)
   fwrite (buffer, 1, size, outfile);
 
   gnutls_x509_crt_deinit (crt);
-  gnutls_x509_privkey_deinit (key);
+  gnutls_privkey_deinit (key);
 }
 
 static void
 generate_signed_certificate (common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt;
-  gnutls_x509_privkey_t key;
+  gnutls_privkey_t key;
   size_t size;
   int result;
-  gnutls_x509_privkey_t ca_key;
+  gnutls_privkey_t ca_key;
   gnutls_x509_crt_t ca_crt;
 
   fprintf (stderr, "Generating a signed certificate...\n");
@@ -872,7 +895,7 @@ generate_signed_certificate (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_sign2 (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
+  result = gnutls_x509_crt_privkey_sign (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -884,14 +907,15 @@ generate_signed_certificate (common_info_st * cinfo)
   fwrite (buffer, 1, size, outfile);
 
   gnutls_x509_crt_deinit (crt);
-  gnutls_x509_privkey_deinit (key);
+  gnutls_privkey_deinit (key);
+  gnutls_privkey_deinit(ca_key);
 }
 
 static void
 generate_proxy_certificate (common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt, eecrt;
-  gnutls_x509_privkey_t key, eekey;
+  gnutls_privkey_t key, eekey;
   size_t size;
   int result;
 
@@ -906,7 +930,7 @@ generate_proxy_certificate (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_sign2 (crt, eecrt, eekey, get_dig (eecrt), 0);
+  result = gnutls_x509_crt_privkey_sign (crt, eecrt, eekey, get_dig (eecrt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -917,8 +941,10 @@ generate_proxy_certificate (common_info_st * cinfo)
 
   fwrite (buffer, 1, size, outfile);
 
+  gnutls_x509_crt_deinit (eecrt);
   gnutls_x509_crt_deinit (crt);
-  gnutls_x509_privkey_deinit (key);
+  gnutls_privkey_deinit (key);
+  gnutls_privkey_deinit (eekey);
 }
 
 static void
@@ -926,8 +952,7 @@ generate_signed_crl (common_info_st * cinfo)
 {
   gnutls_x509_crl_t crl;
   int result;
-  gnutls_x509_privkey_t ca_key;
-  gnutls_privkey_t ca_pkey;
+  gnutls_privkey_t ca_key;
   gnutls_x509_crt_t ca_crt;
 
   fprintf (stderr, "Generating a signed CRL...\n");
@@ -936,23 +961,14 @@ generate_signed_crl (common_info_st * cinfo)
   ca_crt = load_ca_cert (cinfo);
   crl = generate_crl (ca_crt, cinfo);
 
-  result = gnutls_privkey_init(&ca_pkey);
-  if (result < 0)
-    error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (result));
-
-  result = gnutls_privkey_import_x509(ca_pkey, ca_key, 0);
-  if (result < 0)
-    error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (result));
-
   fprintf (stderr, "\n");
-
-  result = gnutls_x509_crl_privkey_sign(crl, ca_crt, ca_pkey, GNUTLS_DIG_SHA1, 0);
+  result = gnutls_x509_crl_privkey_sign(crl, ca_crt, ca_key, SIGN_HASH, 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crl_privkey_sign: %s", gnutls_strerror (result));
 
   print_crl_info (crl, stderr);
 
-  gnutls_privkey_deinit( ca_pkey);
+  gnutls_privkey_deinit( ca_key);
   gnutls_x509_crl_deinit (crl);
 }
 
@@ -962,7 +978,7 @@ update_signed_certificate (common_info_st * cinfo)
   gnutls_x509_crt_t crt;
   size_t size;
   int result;
-  gnutls_x509_privkey_t ca_key;
+  gnutls_privkey_t ca_key;
   gnutls_x509_crt_t ca_crt;
   int days;
   time_t tim = time (NULL);
@@ -985,7 +1001,7 @@ update_signed_certificate (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_sign2 (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
+  result = gnutls_x509_crt_privkey_sign (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -1079,9 +1095,8 @@ gaa_parser (int argc, char **argv)
 
   if ((ret = gnutls_global_init ()) < 0)
     error (EXIT_FAILURE, 0, "global_init: %s", gnutls_strerror (ret));
-
-  if ((ret = gnutls_global_init_extra ()) < 0)
-    error (EXIT_FAILURE, 0, "global_init_extra: %s", gnutls_strerror (ret));
+    
+  pkcs11_common();
 
   memset (&cinfo, 0, sizeof (cinfo));
   cinfo.privkey = info.privkey;
@@ -1640,40 +1655,11 @@ crq_info (void)
   gnutls_x509_crq_deinit (crq);
 }
 
-void
-privkey_info (void)
+static void privkey_info_int (gnutls_x509_privkey_t key)
 {
-  gnutls_x509_privkey_t key;
-  size_t size;
-  int ret;
-  gnutls_datum_t pem;
-  const char *cprint;
-  const char *pass;
-
-  size = fread (buffer, 1, buffer_size - 1, infile);
-  buffer[size] = 0;
-
-  gnutls_x509_privkey_init (&key);
-
-  pem.data = buffer;
-  pem.size = size;
-
-  ret = 0;
-  if (!info.pkcs8)
-    ret = gnutls_x509_privkey_import (key, &pem, info.incert_format);
-
-  /* If we failed to import the certificate previously try PKCS #8 */
-  if (info.pkcs8 || ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR)
-    {
-      if (info.pass)
-        pass = info.pass;
-      else
-        pass = get_pass ();
-      ret = gnutls_x509_privkey_import_pkcs8 (key, &pem,
-                                              info.incert_format, pass, 0);
-    }
-  if (ret < 0)
-    error (EXIT_FAILURE, 0, "import error: %s", gnutls_strerror (ret));
+int ret;
+size_t size;
+const char *cprint;
 
   /* Public key algorithm
    */
@@ -1683,7 +1669,7 @@ privkey_info (void)
 
   cprint = gnutls_pk_algorithm_get_name (ret);
   fprintf (outfile, "%s\n", cprint ? cprint : "Unknown");
-  fprintf (outfile, "\tKey Security Level: %s\n",
+  fprintf (outfile, "\tKey Security Level: %s\n\n",
            gnutls_sec_param_get_name (gnutls_x509_privkey_sec_param (key)));
 
   /* Print the raw public and private keys
@@ -1729,6 +1715,23 @@ privkey_info (void)
           gnutls_free (g.data);
         }
     }
+  else if (ret == GNUTLS_PK_ECC)
+    {
+      gnutls_datum_t y, x, k;
+      gnutls_ecc_curve_t curve;
+
+      ret = gnutls_x509_privkey_export_ecc_raw (key, &curve, &x, &y, &k);
+      if (ret < 0)
+        fprintf (stderr, "Error in key ECC data export: %s\n",
+                 gnutls_strerror (ret));
+      else
+        {
+          print_ecc_pkey (curve, &k, &x, &y);
+          gnutls_free (x.data);
+          gnutls_free (y.data);
+          gnutls_free (k.data);
+        }
+    }
 
   fprintf (outfile, "\n");
 
@@ -1742,6 +1745,46 @@ privkey_info (void)
     {
       fprintf (outfile, "Public Key ID: %s\n", raw_to_string (buffer, size));
     }
+  fprintf (outfile, "\n");
+
+}
+
+void
+privkey_info (void)
+{
+  gnutls_x509_privkey_t key;
+  size_t size;
+  int ret;
+  gnutls_datum_t pem;
+  const char *pass;
+
+  size = fread (buffer, 1, buffer_size - 1, infile);
+  buffer[size] = 0;
+
+  gnutls_x509_privkey_init (&key);
+
+  pem.data = buffer;
+  pem.size = size;
+
+  ret = 0;
+  if (!info.pkcs8)
+    ret = gnutls_x509_privkey_import (key, &pem, info.incert_format);
+
+  /* If we failed to import the certificate previously try PKCS #8 */
+  if (info.pkcs8 || ret == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR)
+    {
+      if (info.pass)
+        pass = info.pass;
+      else
+        pass = get_pass ();
+      ret = gnutls_x509_privkey_import_pkcs8 (key, &pem,
+                                              info.incert_format, pass, 0);
+    }
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "import error: %s", gnutls_strerror (ret));
+
+
+  privkey_info_int (key);
 
   if (info.fix_key != 0)
     {
@@ -1767,7 +1810,8 @@ void
 generate_request (common_info_st * cinfo)
 {
   gnutls_x509_crq_t crq;
-  gnutls_x509_privkey_t key;
+  gnutls_x509_privkey_t xkey;
+  gnutls_pubkey_t pubkey;
   gnutls_privkey_t pkey;
   int ret, ca_status, path_len;
   const char *pass;
@@ -1785,17 +1829,19 @@ generate_request (common_info_st * cinfo)
 
   /* Load the private key.
    */
-  key = load_private_key (0, cinfo);
-  if (!key)
+  pkey = load_private_key (0, cinfo);
+  if (!pkey)
     {
-      key = generate_private_key_int ();
+      xkey = generate_private_key_int ();
 
-      print_private_key (key);
+      print_private_key (xkey);
+
+      ret = gnutls_privkey_import_x509(pkey, xkey, GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE);
+      if (ret < 0)
+        error (EXIT_FAILURE, 0, "privkey_import_x509: %s", gnutls_strerror (ret));
     }
 
-  ret = gnutls_privkey_import_x509(pkey, key, 0);
-  if (ret < 0)
-    error (EXIT_FAILURE, 0, "privkey_import_x509: %s", gnutls_strerror (ret));
+  pubkey = load_public_key_or_import (1, pkey, cinfo);
 
   /* Set the DN.
    */
@@ -1914,11 +1960,11 @@ generate_request (common_info_st * cinfo)
         }
     }
 
-  ret = gnutls_x509_crq_set_key (crq, key);
+  ret = gnutls_x509_crq_set_pubkey (crq, pubkey);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "set_key: %s", gnutls_strerror (ret));
 
-  ret = gnutls_x509_crq_privkey_sign (crq, pkey, GNUTLS_DIG_SHA1, 0);
+  ret = gnutls_x509_crq_privkey_sign (crq, pkey, SIGN_HASH, 0);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "sign: %s", gnutls_strerror (ret));
 
@@ -1926,7 +1972,7 @@ generate_request (common_info_st * cinfo)
 
   gnutls_x509_crq_deinit (crq);
   gnutls_privkey_deinit( pkey);
-  gnutls_x509_privkey_deinit (key);
+  gnutls_pubkey_deinit( pubkey);
 
 }
 
@@ -2310,7 +2356,7 @@ generate_pkcs8 (common_info_st * cinfo)
 
   fprintf (stderr, "Generating a PKCS #8 key structure...\n");
 
-  key = load_private_key (1, cinfo);
+  key = load_x509_private_key (1, cinfo);
 
   if (info.pass)
     password = info.pass;
@@ -2362,7 +2408,7 @@ generate_pkcs12 (common_info_st * cinfo)
 
   fprintf (stderr, "Generating a PKCS #12 structure...\n");
 
-  key = load_private_key (0, cinfo);
+  key = load_x509_private_key (0, cinfo);
   crts = load_cert_list (0, &ncrts, cinfo);
 
   name = get_pkcs12_key_name ();
@@ -2933,6 +2979,22 @@ pubkey_info (gnutls_x509_crt_t crt, common_info_st * cinfo)
           gnutls_free (g.data);
         }
     }
+  else if (ret == GNUTLS_PK_ECC)
+    {
+      gnutls_datum_t x, y;
+      gnutls_ecc_curve_t curve;
+
+      ret = gnutls_pubkey_get_pk_ecc_raw (pubkey, &curve, &x, &y);
+      if (ret < 0)
+        fprintf (stderr, "Error in key ECC data export: %s\n",
+                 gnutls_strerror (ret));
+      else
+        {
+          print_ecc_pkey (curve, NULL, &y, &x);
+          gnutls_free (y.data);
+          gnutls_free (x.data);
+        }
+    }
 
   ret = gnutls_pubkey_get_key_usage (pubkey, &usage);
   if (ret < 0)
