@@ -20,6 +20,7 @@
 
 fail() {
    echo "Failure: $1" >&2
+   kill $PID
    exit 1
 }
 
@@ -38,6 +39,21 @@ launch_server() {
        fi
 }
 
+launch_bare_server() {
+       PARENT=$1;
+       shift;
+       $SERV $* >/dev/null 2>&1 &
+       LOCALPID="$!";
+       trap "[ ! -z \"${LOCALPID}\" ] && kill ${LOCALPID};" 15
+       wait "${LOCALPID}"
+       LOCALRET="$?"
+       if [ "${LOCALRET}" != "0" -a "${LOCALRET}" != "143" ] ; then
+               # Houston, we'v got a problem...
+               echo "Failed to launch server !"
+               kill -10 ${PARENT}
+       fi
+}
+
 wait_server() {
 	trap "kill $1" 1 15 2
 	sleep 2
